@@ -8,10 +8,11 @@ from sqlalchemy.orm.properties import BackRef
 from sqlalchemy import and_, or_
 from sqlalchemy import func
 from sqlalchemy import Table as SATable
-from sqlalchemy import Column, Integer, DateTime, UniqueConstraint
+from sqlalchemy import Column, Integer, Unicode, DateTime, UniqueConstraint
 
 from zope.interface import implements
-from clio.interfaces import IClioModel
+from zope import component
+from clio.interfaces import IClioModel, ICurrentUserId
 
 NEW = 0
 EDITABLE = 1
@@ -32,6 +33,8 @@ NEW_VERSION_STATUSES = (NEW, EDITABLE, UPDATABLE) + DELETED_STATUSES
 
 # any statuses applicable to an edit UI
 EDITABLE_STATUSES = NEW_VERSION_STATUSES + (PUBLISHED,)
+# any statuses that count as as actual (not-deleted) in the edit UI
+ACTUAL_STATUSES = (NEW, EDITABLE, UPDATABLE, PUBLISHED)
 
 class ClioError(Exception):
     pass
@@ -61,8 +64,13 @@ def Table(name, metadata, *args, **kw):
                    Column('id', Integer, primary_key=True),
                    Column('code', Integer, nullable=False, index=True),
                    Column('status', Integer, nullable=False),
-                   Column('workflow_timestamp', DateTime, nullable=False),
-                   Column('creation_timestamp', DateTime, nullable=False),
+                   Column('created_timestamp', DateTime, nullable=False),
+                   Column('published_start_timestamp', DateTime, nullable=True,
+                          index=True),
+                   Column('published_end_timestamp', DateTime, nullable=True,
+                          index=True),
+                   Column('published_userid', Unicode(64), nullable=True),
+                   Column('changed_userid', Unicode(64), nullable=True),
                    UniqueConstraint('code', 'status', name='workflow'),
                    *args, **kw)
 
@@ -136,9 +144,12 @@ class Model(object):
     def __init__(self, code):
         self.code = code
         self.status = NEW
-        self.workflow_timestamp = self._now()
-        self.creation_timestamp = self.workflow_timestamp
- 
+        self.created_timestamp = self._now()
+        self.published_start_timestamp = None
+        self.published_end_timestamp = None
+        self.edited_userid = None
+        self.published_userid = None
+        
     def publish(self):
         if self.is_archived():
             raise PublishError("Cannot publish an archived object.")
@@ -157,14 +168,17 @@ class Model(object):
                     "Cannot publish object that has "
                     "an unpublished parent object.")
             self.status = PUBLISHED
-            self.workflow_timestamp = self._now()
+            self.published_start_timestamp = self._now()
+            self.published_userid = self._get_userid()
             self._publish_related()
             return self
         elif self.status == EDITABLE:
             published = self._version(PUBLISHED_UNDER_EDIT)
-            published._archive()
+            timestamp = self._now()
+            published._archive(timestamp)
             self.status = PUBLISHED
-            self.workflow_timestamp = self._now()
+            self.published_start_timestamp = timestamp
+            self.published_userid = self._get_userid()
             self._publish_related()
             return self
         elif self.status == UPDATABLE:
@@ -172,13 +186,16 @@ class Model(object):
             # version over to this version
             published = self._version(PUBLISHED_UNDER_UPDATE)
             self._move_relations(published)
-            published._archive()
+            timestamp = self._now()
+            published._archive(timestamp)
             self.status = PUBLISHED
-            self.workflow_timestamp = self._now()
+            self.published_start_timestamp = timestamp
+            self.published_userid = self._get_userid()
             return self
         elif self.status in DELETED_STATUSES:
             published = self._version(PUBLISHED_UNDER_DELETE)
-            published._archive()
+            published.published_userid = self._get_userid()
+            published._archive(self._now())
             self._publish_related()
             session = object_session(self)
             session.delete(self)
@@ -205,9 +222,8 @@ class Model(object):
         if self.status != PUBLISHED:
             raise EditError(
                 "Cannot edit object of unknown status: %s" % self.status)
-        result = self._create_edit_copy()
+        result = self._create_edit_copy() 
         self.status = PUBLISHED_UNDER_EDIT
-        result.workflow_timestamp = self._now()
         session = object_session(self)
         session.add(result)
         return result
@@ -232,7 +248,6 @@ class Model(object):
                 "Cannot update object of unknown status: %s" % self.status)
         result = self._create_update_copy()
         self.status = PUBLISHED_UNDER_UPDATE
-        result.workflow_timestamp = self._now()
         session = object_session(self)
         session.add(result)
         return result
@@ -267,7 +282,6 @@ class Model(object):
             session.add(result)
             self.status = PUBLISHED_UNDER_DELETE
             result.status = DELETED
-            result.workflow_timestamp = self._now()
             result._delete_related()
             return result
         else:            
@@ -358,14 +372,23 @@ class Model(object):
             elif obj.status in DELETED_STATUSES:
                 deleted.append(obj)
         return ComparisonResult(editable, deleted, unchanged)        
-        
+
+    def mark_changed(self):
+        self.changed_userid = self._get_userid()
+            
     # PRIVATE
     def _now(self):
         return datetime.now()
+    
+    def _get_userid(self):
+        try:
+            return component.getUtility(ICurrentUserId)()
+        except component.ComponentLookupError:
+            return u'unknown'
 
-    def _archive(self):
+    def _archive(self, timestamp):
         self.status = self._get_next_archive_status()
-        self.workflow_timestamp = self._now()
+        self.published_end_timestamp = timestamp
         return self
 
     def _get_relation_props(self, *directions):
@@ -424,9 +447,13 @@ class Model(object):
         for prop in self._get_other_props():
             value = getattr(self, prop.key)
             setattr(result, prop.key, value)
+        result.published_start_timestamp = None
+        result.published_end_timestamp = None
+        result.created_timestamp = result._now()
         del result.id
-        result.creation_timestamp = self._now()
         result.status = UPDATABLE
+        result.changed_userid = None
+        result.published_userid = None
         return result
 
     def _create_update_copy_onto(self, target):
@@ -440,11 +467,17 @@ class Model(object):
                 value = getattr(self, prop.key)
                 setattr(result, prop.key, value)
         result.status = UPDATABLE
+        result.published_start_timestamp = None
+        result.published_end_timestamp = None
+        result.created_timestamp = result._now()
+        result.changed_userid = None
+        result.published_userid = None
         return result
 
     def _create_edit_copy(self):
         result = self._create_update_copy()
         result.status = EDITABLE
+
         # this has to be done *after* the basic properties of result
         # are set otherwise relations don't get set up properly
         for prop in self._get_relation_props(ONETOMANY, MANYTOMANY):
@@ -504,7 +537,7 @@ class Model(object):
                 #    orig_values.remove(related)
                 #    values.append(related)
                 if related.status == PUBLISHED:
-                    values.append(related._create_edit_copy())
+                    values.append(related._create_edit_cpy())
                     related.status = PUBLISHED_UNDER_EDIT
         return result
         
@@ -529,4 +562,3 @@ class Model(object):
             if version is not None:
                 return version
         return None
-
