@@ -2,6 +2,9 @@ import unittest, doctest
 from datetime import datetime, timedelta
 
 import clio
+from clio.interfaces import ICurrentUserId
+
+from zope import component
 
 from sqlalchemy import MetaData, create_engine
 from sqlalchemy import Table, Column, Integer, Unicode
@@ -127,20 +130,35 @@ def now():
     delta *= 100000
     return _first_dt + delta
 
+def setup_clean_database():
+    engine = create_engine('mysql:///clio_tests', echo=False)
+    metadata.reflect(engine)
+    for table in reversed(metadata.sorted_tables):
+        engine.execute(table.delete())
+    for code in [a_code, b_code, c_code, d_code]:
+        code.reinitialize()
+    return engine
+
+userid_count = 0
+
+def get_userid():
+    global userid_count
+    userid = u'userid_%s' % userid_count
+    userid_count += 1
+    return userid
+
 class ClioTestCase(unittest.TestCase):
     def setUp(self):
-        engine = create_engine('mysql:///clio_tests', echo=False)
-        metadata.create_all(engine)
+        engine = setup_clean_database()
         Session = sessionmaker(bind=engine)
         self.session = Session()
-        self.session.query(B).delete()
-        self.session.query(A).delete()
-        self.session.execute(c_d.delete())
-        self.session.query(C).delete()
-        self.session.query(D).delete()
         self.session.flush()
         self.session.commit()
-        
+        global userid_count
+        userid_count = 0
+        component.provideUtility(
+            get_userid, ICurrentUserId)
+
     def tearDown(self):
         self.session.commit()
 
@@ -197,9 +215,10 @@ class ClioTestCase(unittest.TestCase):
         one = A(a_code(self.session), u"One")
         after = now()
 
-        self.assert_(before < one.creation_timestamp < after)
-        self.assert_(before < one.workflow_timestamp < after)
-
+        self.assert_(before < one.created_timestamp < after)
+        self.assertEquals(None, one.published_start_timestamp)
+        self.assertEquals(None, one.published_end_timestamp)
+        
     def test_a_new_change(self):
         """We can change new A.
         """
@@ -210,6 +229,13 @@ class ClioTestCase(unittest.TestCase):
         one.message = u"One New Message"
         self.session.commit()
 
+    def test_a_mark_changed(self):
+        one = A(a_code(self.session), u'One')
+        self.session.add(one)
+        self.assertEquals(None, one.changed_userid)
+        one.mark_changed()
+        self.assertEquals(u'userid_0', one.changed_userid)
+        
     def test_a_publish(self):
         """We can publish a new A.
         """
@@ -231,10 +257,22 @@ class ClioTestCase(unittest.TestCase):
         after_publish = now()
 
         self.assert_(
-            before_create < one.creation_timestamp < after_create)
+            before_create < one.created_timestamp < after_create)
         self.assert_(
-            before_publish < one.workflow_timestamp < after_publish)
+            before_publish < one.published_start_timestamp < after_publish)
+        self.assertEquals(None, one.published_end_timestamp)
 
+    def test_a_publish_userid(self):
+        """We can publish a new A.
+        """
+        one = A(a_code(self.session), u'One')
+        self.session.add(one)
+        self.assertEquals(None, one.published_userid)
+        
+        one = one.publish()
+
+        self.assertEquals(u'userid_0', one.published_userid)
+    
     def test_cannot_change_published(self):
         pass # to be implemented
         
@@ -269,13 +307,25 @@ class ClioTestCase(unittest.TestCase):
         one_editable = one.edit()
         after_edit = now()        
 
-        self.assert_(before_create < one.creation_timestamp < after_create)
-        self.assert_(before_publish < one.workflow_timestamp < after_publish)
+        self.assert_(before_create < one.created_timestamp < after_create)
+        self.assert_(before_publish < one.published_start_timestamp < after_publish)
+        self.assertEquals(None, one.published_end_timestamp)
         self.assert_(
-            before_edit < one_editable.workflow_timestamp < after_edit)
-        self.assert_(
-            before_edit < one_editable.creation_timestamp < after_edit)
+            before_edit < one_editable.created_timestamp < after_edit)
+        self.assertEquals(None, one_editable.published_start_timestamp)
+        self.assertEquals(None, one_editable.published_end_timestamp)
 
+    def test_a_edit_userid(self):
+        """We can edit a published A.
+        """
+        one = A(a_code(self.session), u'One')
+        self.session.add(one)
+        one = one.publish()
+        self.session.commit()
+
+        one_editable = one.edit()
+        self.assertEquals(None, one_editable.published_userid)
+    
     def test_a_update(self):
         """We can update a published A.
 
@@ -309,13 +359,14 @@ class ClioTestCase(unittest.TestCase):
         one_updatable = one.update()
         after_update = now()        
 
-        self.assert_(before_create < one.creation_timestamp < after_create)
-        self.assert_(before_publish < one.workflow_timestamp < after_publish)
+        self.assert_(before_create < one.created_timestamp < after_create)
+        self.assert_(before_publish < one.published_start_timestamp < after_publish)
+        self.assertEquals(None, one.published_end_timestamp)
         self.assert_(
-            before_update < one_updatable.workflow_timestamp < after_update)
-        self.assert_(
-            before_update < one_updatable.creation_timestamp < after_update)
-
+            before_update < one_updatable.created_timestamp < after_update)
+        self.assertEquals(None, one_updatable.published_start_timestamp)
+        self.assertEquals(None, one_updatable.published_end_timestamp)
+ 
     def test_a_publish_editable(self):
         """We can publish an edited A.
         """
@@ -331,8 +382,9 @@ class ClioTestCase(unittest.TestCase):
         before_publish = now()
         one = one_editable.publish()
         after_publish = now()
-        self.assert_(before_publish < one.workflow_timestamp < after_publish)
-
+        self.assert_(before_publish < one.published_start_timestamp < after_publish)
+        self.assertEquals(None, one.published_end_timestamp)
+ 
     def test_a_publish_updatable(self):
         """We can publish an updatable A.
 
@@ -417,14 +469,29 @@ class ClioTestCase(unittest.TestCase):
         before_publish2 = now()
         one3 = one_editable3.publish()
         after_publish2 = now()
+ 
+        self.assert_(
+            before_publish1 < one2.published_start_timestamp < after_publish1)
+        self.assert_(
+            before_publish2 < one3.published_start_timestamp < after_publish2)
+        self.assert_(
+            before_publish2 < one2.published_end_timestamp < after_publish2)
 
-        self.assert_(
-            before_publish1 < one1.workflow_timestamp < after_publish1)
-        self.assert_(
-            before_publish2 < one2.workflow_timestamp < after_publish2)
-        self.assert_(
-            before_publish2 < one3.workflow_timestamp < after_publish2)
+    def test_a_archive_userid(self):
+        """When we publish an edited A, the previous version is archived.
+        """
+        one1 = self._get_published()
+        one_editable2 = one1.edit()
+        one_editable2.message = u'One 2'
+        one2 = one_editable2.publish()
+        one_editable3 = one2.edit()
+        one_editable3.message = u'One 3'
+        one3 = one_editable3.publish()
 
+        self.assertEquals(u'userid_0', one1.published_userid)
+        self.assertEquals(u'userid_1', one2.published_userid)
+        self.assertEquals(u'userid_2', one3.published_userid)
+    
     def test_a_archive_updatable(self):
         """When we publish an updated, the previous version is archived.
 
@@ -1232,9 +1299,8 @@ class CopyModelTestCase(unittest.TestCase):
         self.assertEquals(2, len(list(new.bs_editable)))
 
 def readmeSetUp(test):
-    pass
+    setup_clean_database()
 
-    
 def test_suite():
     optionflags=(doctest.ELLIPSIS+
                  doctest.NORMALIZE_WHITESPACE+
